@@ -1,4 +1,36 @@
-#include "includes.h"
+/*
+    Glurp - A GTK+ client for Music Player Daemon
+    Copyright (C) 2004, 2005 Andrej Kacian
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    http://musicpd.org/glurp.shtml
+
+*/
+
+#include <glib.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+#include <stdlib.h>
+
+#include "structs.h"
+#include "support.h"
+#include "comm.h"
+#include "gui.h"
+#include "player.h"
+#include "conf.h"
 
 extern GladeXML *guixml, *configxml, *plxml, *addxml, *streamxml;
 extern GlurpState *glurp;
@@ -9,9 +41,23 @@ void on_ui_quit(GtkWidget *widget, gpointer user_data) {
 
   if( glurp->conn ) glurp_disconnect();
 
+  config_save();
+
   gtk_main_quit();
 }
 
+gboolean on_window_moved(GtkWidget *widget, GdkEventConfigure *event, gpointer data) {
+  debug("Storing window position and size (%dx%d+%d+%d)", event->width, event->height, event->x, event->y);
+
+  glurp->config->pos_x = event->x;
+  glurp->config->pos_y = event->y;
+
+  glurp->config->width = event->width;
+  glurp->config->height = event->height;
+
+  return FALSE;
+}
+
 /* called when config button is pressed in main window */
 gboolean on_ui_press_config(GtkWidget *widget, gpointer user_data) {
   GtkWidget *window_config;
@@ -23,12 +69,12 @@ gboolean on_ui_press_config(GtkWidget *widget, gpointer user_data) {
 
   debug("Displaying config window...");
 
-  configxml = glade_xml_new(glade_path(), "window_config", NULL);
+  configxml = glade_xml_new(glade_path(), "glurp_window_config", NULL);
   glade_xml_signal_autoconnect(configxml);
 
   populate_config();
 
-  window_config = glade_xml_get_widget(configxml, "window_config");
+  window_config = glade_xml_get_widget(configxml, "glurp_window_config");
 
   title_print(window_config, "Config");
 
@@ -38,7 +84,7 @@ gboolean on_ui_press_config(GtkWidget *widget, gpointer user_data) {
 }
 
 gboolean on_button_config_cancel_clicked(GtkWidget *widget, gpointer user_data) {
-  GtkWidget *window_config = glade_xml_get_widget(configxml, "window_config");
+  GtkWidget *window_config = glade_xml_get_widget(configxml, "glurp_window_config");
   
   debug("Cancel pressed, destroying config window...");
   gtk_widget_destroy(window_config);
@@ -65,7 +111,7 @@ gboolean on_button_config_ok_clicked(GtkWidget *widget, gpointer user_data) {
 
   config_save();
 
-  window_config = glade_xml_get_widget(configxml, "window_config");
+  window_config = glade_xml_get_widget(configxml, "glurp_window_config");
   gtk_widget_destroy(window_config);
 
   statusbar_print("Config saved...");
@@ -144,7 +190,7 @@ gboolean on_ui_progress_change_start(GtkWidget *widget, gpointer user_data) {
 }
 
 gboolean on_ui_playlist_clicked(GtkWidget *widget, gpointer user_data) {
-  if( !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)) )
+  if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) )
     hide_gui_playlist();
   else
     show_gui_playlist();
@@ -267,7 +313,7 @@ gboolean on_ui_playlist_row_activated(GtkTreeView *treeview, GtkTreePath *tp, Gt
   return FALSE;
 }
 
-gboolean on_ui_playlists_pressed(GtkWidget *widget, gpointer user_data) {
+gboolean on_ui_playlists_clicked(GtkWidget *widget, gpointer user_data) {
   GtkWidget *window_playlist;
 
   if(plxml) {
@@ -280,7 +326,7 @@ gboolean on_ui_playlists_pressed(GtkWidget *widget, gpointer user_data) {
     return FALSE;
   }
 
-  plxml = glade_xml_new(glade_path(), "window_playlist_list", NULL);
+  plxml = glade_xml_new(glade_path(), "glurp_window_playlist_list", NULL);
   glade_xml_signal_autoconnect(plxml);
 
   get_playlist_list();
@@ -289,7 +335,7 @@ gboolean on_ui_playlists_pressed(GtkWidget *widget, gpointer user_data) {
 
   populate_gui_playlist_list();
 
-  window_playlist = glade_xml_get_widget(plxml, "window_playlist_list");
+  window_playlist = glade_xml_get_widget(plxml, "glurp_window_playlist_list");
 
   title_print(window_playlist, "Playlists");
 
@@ -301,7 +347,7 @@ gboolean on_ui_playlists_pressed(GtkWidget *widget, gpointer user_data) {
 }
 
 gboolean on_ui_playlist_close(GtkWidget *widget, gpointer user_data) {
-  GtkWidget *w = glade_xml_get_widget(plxml, "window_playlist_list");
+  GtkWidget *w = glade_xml_get_widget(plxml, "glurp_window_playlist_list");
 
   debug("Destroying playlist list window.");
   gtk_widget_destroy(w);
@@ -339,7 +385,7 @@ gboolean on_ui_playlist_list_row_activated(GtkTreeView *treeview, GtkTreePath *t
   g_free(name);
 
   debug("Destroying playlist list window.");
-  gtk_widget_destroy(glade_xml_get_widget(plxml, "window_playlist_list"));
+  gtk_widget_destroy(glade_xml_get_widget(plxml, "glurp_window_playlist_list"));
 
   playlists_window_destroyed();
 
@@ -356,7 +402,7 @@ gboolean on_ui_playlist_load(GtkWidget *widget, gpointer user_data) {
 
   if( !glurp->conn ) {
     debug("We're not connected, how can this be? Just closing the window");
-    gtk_widget_destroy(glade_xml_get_widget(configxml, "window_playlist_list"));
+    gtk_widget_destroy(glade_xml_get_widget(configxml, "glurp_window_playlist_list"));
     return FALSE;
   }
 
@@ -376,7 +422,7 @@ gboolean on_ui_playlist_load(GtkWidget *widget, gpointer user_data) {
   g_free(name);
 
   debug("Destroying playlist list window.");
-  gtk_widget_destroy(glade_xml_get_widget(plxml, "window_playlist_list"));
+  gtk_widget_destroy(glade_xml_get_widget(plxml, "glurp_window_playlist_list"));
 
   playlists_window_destroyed();
 
@@ -393,7 +439,7 @@ gboolean on_ui_playlist_delete(GtkWidget *widget, gpointer user_data) {
 
   if( !glurp->conn ) {
     debug("We're not connected, how can this be? Just closing the window");
-    gtk_widget_destroy(glade_xml_get_widget(configxml, "window_playlist_list"));
+    gtk_widget_destroy(glade_xml_get_widget(configxml, "glurp_window_playlist_list"));
     return FALSE;
   }
 
@@ -409,7 +455,6 @@ gboolean on_ui_playlist_delete(GtkWidget *widget, gpointer user_data) {
   gtk_tree_model_get(tm, &iter, 0, &name, -1);
 
   mpd_sendRmCommand(glurp->conn, name);
-  g_free(name);
 
   mpd_finishCommand(glurp->conn);
 
@@ -419,6 +464,7 @@ gboolean on_ui_playlist_delete(GtkWidget *widget, gpointer user_data) {
   }
 
   debug("Playlist '%s' deleted", name);
+  g_free(name);
   get_playlist_list();
   populate_gui_playlist_list();
 
@@ -495,7 +541,7 @@ gboolean on_ui_playlist_list_cursor_changed(GtkWidget *widget, gpointer user_dat
 
   if( !glurp->conn ) {
     debug("We're not connected, how can this be? Just closing the window");
-    gtk_widget_destroy(glade_xml_get_widget(configxml, "window_playlist_list"));
+    gtk_widget_destroy(glade_xml_get_widget(configxml, "glurp_window_playlist_list"));
     return FALSE;
   }
 
@@ -723,7 +769,7 @@ gboolean on_menu_pl_remove_crop_activate(GtkWidget *widget, gpointer data) {
 }
 
 gboolean on_ui_add_close_clicked(GtkWidget *widget, gpointer data) {
-  GtkWidget *window_add = glade_xml_get_widget(addxml, "window_add");
+  GtkWidget *window_add = glade_xml_get_widget(addxml, "glurp_window_add");
 
   g_object_ref(G_OBJECT(glurp->gui_addtree));
 
@@ -799,11 +845,11 @@ gboolean on_ui_add_add_clicked(GtkWidget *widget, gpointer data) {
 
 gboolean on_ui_repeat_clicked(GtkWidget *widget, gpointer user_data) {
   if( !CONNECTED ) {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(widget), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), FALSE);
     return FALSE;
   }
 
-  if( !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)) ) {
+  if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ) {
     statusbar_print("Repeat Off");
     debug("Repeat Off");
     mpd_sendRepeatCommand(glurp->conn, 0);
@@ -831,11 +877,11 @@ gboolean on_ui_repeat_clicked(GtkWidget *widget, gpointer user_data) {
 
 gboolean on_ui_random_clicked(GtkWidget *widget, gpointer user_data) {
   if( !CONNECTED ) {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(widget), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), FALSE);
     return FALSE;
   }
 
-  if( !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)) ) {
+  if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ) {
     statusbar_print("Random Off");
     debug("Random Off");
     mpd_sendRandomCommand(glurp->conn, 0);
@@ -867,8 +913,24 @@ gboolean on_ui_add_row_expanded(GtkTreeView *tv, GtkTreeIter *iter, GtkTreePath
   return FALSE;
 }
 
+gboolean on_ui_add_find_clicked( GtkWidget *widget, gpointer data) {
+	
+  if( !CONNECTED ) {
+    debug("Not connected, ignoring.");
+    statusbar_print("Not connected, cannot add");
+    return FALSE;
+  }
+  debug("Search started");
+  
+  populate_gui_add_search_tree();
+  
+  return TRUE;
+  
+}
+
 gboolean on_menu_add_file_activate(GtkWidget *widget, gpointer data) {
   GtkWidget *window_add;
+  GtkComboBox* combo_type;
 
   if( !CONNECTED ) {
     debug("Not connected, ignoring.");
@@ -881,11 +943,16 @@ gboolean on_menu_add_file_activate(GtkWidget *widget, gpointer data) {
     return FALSE;
   }
 
-  addxml = glade_xml_new(glade_path(), "window_add", NULL);
+  debug("Add menu");
+
+  addxml = glade_xml_new(glade_path(), "glurp_window_add", NULL);
   glade_xml_signal_autoconnect(addxml);
 
-  window_add = glade_xml_get_widget(addxml, "window_add");
-
+  window_add = glade_xml_get_widget(addxml, "glurp_window_add");
+  
+  combo_type = GTK_COMBO_BOX(glade_xml_get_widget(addxml, "combo_add_find_type"));
+  gtk_combo_box_set_active(combo_type, 0);
+  
   create_gui_add_tree();
 
   debug("Populating 'add' treeview...");
@@ -893,6 +960,8 @@ gboolean on_menu_add_file_activate(GtkWidget *widget, gpointer data) {
 
   title_print(window_add, "Add songs to playlist");
 
+  gtk_widget_grab_focus(GTK_WIDGET(glade_xml_get_widget(addxml, "entry_add_find_what")));
+
   gtk_widget_show(window_add);
 
   return FALSE;
@@ -916,14 +985,14 @@ gboolean on_menu_add_url_activate(GtkWidget *widget, gpointer data) {
     return FALSE;
   }
 
-  streamxml = glade_xml_new(glade_path(), "window_stream", NULL);
+  streamxml = glade_xml_new(glade_path(), "glurp_window_stream", NULL);
   glade_xml_signal_autoconnect(streamxml);
 
   create_stream_liststore();
 
   populate_stream_liststore();
 
-  window_stream = glade_xml_get_widget(streamxml, "window_stream");
+  window_stream = glade_xml_get_widget(streamxml, "glurp_window_stream");
 
   title_print(window_stream, "Add a stream to playlist");
 
@@ -938,7 +1007,7 @@ gboolean on_stream_destroy(GtkWidget *widget, gpointer user_data) {
 }
 
 gboolean on_ui_stream_close_clicked(GtkWidget *widget, gpointer user_data) {
-  GtkWidget *window_stream = glade_xml_get_widget(streamxml, "window_stream");
+  GtkWidget *window_stream = glade_xml_get_widget(streamxml, "glurp_window_stream");
   
   debug("Close pressed, destroying stream window...");
   gtk_widget_destroy(window_stream);
@@ -947,7 +1016,7 @@ gboolean on_ui_stream_close_clicked(GtkWidget *widget, gpointer user_data) {
 }
 
 gboolean on_ui_stream_add_clicked(GtkWidget *widget, gpointer user_data) {
-  GtkWidget *window_stream = glade_xml_get_widget(streamxml, "window_stream");
+  GtkWidget *window_stream = glade_xml_get_widget(streamxml, "glurp_window_stream");
   gchar *url = get_selected_stream();
 
   if( !url || !g_utf8_strlen(url, -1) ) {
@@ -1061,7 +1130,7 @@ gboolean on_ui_playlist_button_press(GtkWidget *widget, GdkEventButton *event, g
   }
 
   debug("Displaying playlist popup menu");
-  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "menu_playlist")), NULL, NULL, NULL, NULL, event->button, event->time);
+  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "glurp_menu_playlist")), NULL, NULL, NULL, NULL, event->button, event->time);
 
   return TRUE;
 }
@@ -1108,7 +1177,7 @@ gboolean on_ui_add_button_press(GtkWidget *widget, GdkEventButton *event, gpoint
   }
 
   debug("Displaying add popup menu");
-  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "menu_db")), NULL, NULL, NULL, NULL, event->button, event->time);
+  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "glurp_menu_db")), NULL, NULL, NULL, NULL, event->button, event->time);
   return TRUE;
 }
 
@@ -1181,7 +1250,7 @@ gboolean on_pmenu_db_info(GtkWidget *widget, gpointer data) {
   return FALSE;
 }
 
-gboolean on_ui_pl_shuffle_clicked(GtkWidget *widget, gpointer data) {
+gboolean on_pmenu_playlist_shuffle_activate(GtkWidget *widget, gpointer data) {
   if( !CONNECTED ) {
     debug("Not connected");
     return FALSE;
@@ -1200,38 +1269,38 @@ gboolean on_ui_pl_shuffle_clicked(GtkWidget *widget, gpointer data) {
   return FALSE;
 }
 
-gboolean on_toggletoolbutton_pl_add_toggled(GtkWidget *widget, gpointer data) {
-  if( !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)) ) return FALSE;
+gboolean on_togglebutton_pl_add_toggled(GtkWidget *widget, gpointer data) {
+  if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ) return FALSE;
 
   debug("Add button clicked");
-  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "menu_pl_add")), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
+  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "glurp_menu_pl_add")), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
   return FALSE;
 }
 
 gboolean on_menu_pl_add_deactivate(GtkWidget *widget, gpointer data) {
   debug("Add menu hidden, deactivating togglebutton");
-  gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "toggletoolbutton_pl_add")), FALSE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_pl_add")), FALSE);
   return FALSE;
 }
 
-gboolean on_toggletoolbutton_pl_remove_toggled(GtkWidget *widget, gpointer data) {
-  if( !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)) ) return FALSE;
+gboolean on_togglebutton_pl_remove_toggled(GtkWidget *widget, gpointer data) {
+  if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ) return FALSE;
 
   debug("Remove button clicked");
-  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "menu_pl_remove")), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
+  gtk_menu_popup(GTK_MENU(glade_xml_get_widget(guixml, "glurp_menu_pl_remove")), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
   return FALSE;
 }
 
 gboolean on_menu_pl_remove_deactivate(GtkWidget *widget, gpointer data) {
   debug("Remove menu hidden, deactivating togglebutton");
-  gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "toggletoolbutton_pl_remove")), FALSE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_pl_remove")), FALSE);
   return FALSE;
 }
 
 gboolean on_outputs_toggled(GtkWidget *widget, gpointer data) {
   GtkMenu *menu = populate_outputs_menu();
 
-  if( !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)) ) return FALSE;
+  if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ) return FALSE;
 
   debug("Outputs button clicked");
 
@@ -1243,7 +1312,7 @@ gboolean on_outputs_toggled(GtkWidget *widget, gpointer data) {
 
 gboolean on_menu_outputs_deactivate(GtkWidget *widget, gpointer data) {
   debug("Outputs menu hidden, deactivating togglebutton");
-  gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "toggletoolbutton_outputs")), FALSE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_outputs")), FALSE);
   return FALSE;
 }
 
