@@ -1,4 +1,36 @@
-#include "includes.h"
+/*
+    Glurp - A GTK+ client for Music Player Daemon
+    Copyright (C) 2004, 2005 Andrej Kacian
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    http://musicpd.org/glurp.shtml
+
+*/
+
+#include <glib.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+#include <stdlib.h>
+
+#include "structs.h"
+#include "support.h"
+#include "comm.h"
+#include "conf.h"
+#include "player.h"
+#include "gui.h"
 
 extern GladeXML *guixml, *configxml, *plxml, *addxml, *streamxml;
 extern GlurpState *glurp;
@@ -31,81 +63,86 @@ gboolean glurp_init_gui() {
 
   /* put correct icons to player control buttons */
   /* prev */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_prev")), gtk_image_new_from_file(DATADIR "pixmaps/player-prev.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_prev")), gtk_image_new_from_file(DATADIR "pixmaps/player-prev.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_prev"));
   /* play */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_play")), gtk_image_new_from_file(DATADIR "pixmaps/player-play.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_play")), gtk_image_new_from_file(DATADIR "pixmaps/player-play.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_play"));
   /* pause */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_pause")), gtk_image_new_from_file(DATADIR "pixmaps/player-pause.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_pause")), gtk_image_new_from_file(DATADIR "pixmaps/player-pause.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_pause"));
   /* stop */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_stop")), gtk_image_new_from_file(DATADIR "pixmaps/player-stop.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_stop")), gtk_image_new_from_file(DATADIR "pixmaps/player-stop.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_stop"));
   /* next */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_next")), gtk_image_new_from_file(DATADIR "pixmaps/player-next.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_next")), gtk_image_new_from_file(DATADIR "pixmaps/player-next.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_next"));
 
   /* connect/disconnect */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_server_disconnect")), gtk_image_new_from_file(DATADIR "pixmaps/offline.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_server_disconnect")), gtk_image_new_from_file(DATADIR "pixmaps/offline.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_server_disconnect"));
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_server_connect")), gtk_image_new_from_file(DATADIR "pixmaps/online.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_server_connect")), gtk_image_new_from_file(DATADIR "pixmaps/online.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_server_connect"));
 
   /* config */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_config")), gtk_image_new_from_file(DATADIR "pixmaps/config.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_config")), gtk_image_new_from_file(DATADIR "pixmaps/config.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "button_config"));
 
   /* playlist show/hide */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_playlist")), gtk_image_new_from_file(DATADIR "pixmaps/playlist.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "togglebutton_playlist")), gtk_image_new_from_file(DATADIR "pixmaps/playlist.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "togglebutton_playlist"));
 
   /* repeat */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), gtk_image_new_from_file(DATADIR "pixmaps/player-repeat.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "togglebutton_repeat")), gtk_image_new_from_file(DATADIR "pixmaps/player-repeat.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "togglebutton_repeat"));
 
   /* random */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), gtk_image_new_from_file(DATADIR "pixmaps/player-random.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "togglebutton_random")), gtk_image_new_from_file(DATADIR "pixmaps/player-random.png"));
   gtk_widget_show_all(glade_xml_get_widget(guixml, "togglebutton_random"));
 
-  /* playlist shuffle */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_pl_shuffle")), gtk_image_new_from_file(DATADIR "pixmaps/player-shuffle.png"));
-  gtk_widget_show_all(glade_xml_get_widget(guixml, "button_pl_shuffle"));
-
   /* add */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "toggletoolbutton_pl_add")), gtk_image_new_from_file(DATADIR "pixmaps/add.png"));
-  gtk_widget_show_all(glade_xml_get_widget(guixml, "toggletoolbutton_pl_add"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "togglebutton_pl_add")), gtk_image_new_from_file(DATADIR "pixmaps/add.png"));
+  gtk_widget_show_all(glade_xml_get_widget(guixml, "togglebutton_pl_add"));
 
   /* remove */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "toggletoolbutton_pl_remove")), gtk_image_new_from_file(DATADIR "pixmaps/remove.png"));
-  gtk_widget_show_all(glade_xml_get_widget(guixml, "toggletoolbutton_pl_remove"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "togglebutton_pl_remove")), gtk_image_new_from_file(DATADIR "pixmaps/remove.png"));
+  gtk_widget_show_all(glade_xml_get_widget(guixml, "togglebutton_pl_remove"));
 
   /* playlists */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "button_pl_file")), gtk_image_new_from_file(DATADIR "pixmaps/playlists.png"));
-  gtk_widget_show_all(glade_xml_get_widget(guixml, "button_pl_file"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "button_playlists")), gtk_image_new_from_file(DATADIR "pixmaps/playlists.png"));
+  gtk_widget_show_all(glade_xml_get_widget(guixml, "button_playlists"));
 
   /* outputs */
-  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(glade_xml_get_widget(guixml, "toggletoolbutton_outputs")), gtk_image_new_from_file(DATADIR "pixmaps/outputs.png"));
+  gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(guixml, "togglebutton_outputs")), gtk_image_new_from_file(DATADIR "pixmaps/outputs.png"));
 
   /* optionally show playlist */
   show_gui_playlist();
   if(glurp->config->playlist_vis_on_start) {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_playlist")), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_playlist")), TRUE);
   } else {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_playlist")), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_playlist")), FALSE);
     hide_gui_playlist();
   }
 
   gui_set_disconnected();
 
-  window_main = glade_xml_get_widget( guixml, "window_main" );
+  window_main = glade_xml_get_widget( guixml, "glurp_window_main" );
 
   title_print(window_main, NULL);
 
-  gui_create_tooltips();
-
   gui_refresh_playlist_columns();
 
+/* a futile attempt to restore saved position
+  if( glurp->config->pos_x != -11000 && glurp->config->pos_y != -11000 )
+    gtk_window_move(GTK_WINDOW(window_main), glurp->config->pos_x, glurp->config->pos_y);
+*/
+
+  /* restore window size if requested */
+  if( glurp->config->width >= 0 && glurp->config->height >= 0 && glurp->config->save_size )
+    gtk_window_resize(GTK_WINDOW(window_main), glurp->config->width, glurp->config->height);
+
+  gtk_window_set_icon(GTK_WINDOW(window_main), gdk_pixbuf_new_from_file(DATADIR "pixmaps/media-audiofile.png", NULL));
+
   debug("Showing main window...");
   gtk_widget_show(window_main);
 
@@ -128,6 +165,10 @@ void populate_config() {
   if(glurp->config->autoconnect) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
+  w = glade_xml_get_widget(configxml, "checkbutton_save_size");
+  if(glurp->config->autoconnect) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
+
   w = glade_xml_get_widget(configxml, "checkbutton_playlist_vis");
   if(glurp->config->playlist_vis_on_start) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
@@ -136,31 +177,31 @@ void populate_config() {
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), glurp->config->refresh_rate);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_filename");
-  if(glurp->config->vis_filename) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_FILENAME]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_artist");
-  if(glurp->config->vis_artist) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_ARTIST]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_title");
-  if(glurp->config->vis_title) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_TITLE]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_album");
-  if(glurp->config->vis_album) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_ALBUM]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_name");
-  if(glurp->config->vis_name) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_NAME]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_pos");
-  if(glurp->config->vis_pos) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_POS]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_id");
-  if(glurp->config->vis_id) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
+  if(glurp->config->playlist_columns[PL_ID]) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 1);
   else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), 0);
 
 } 
@@ -198,26 +239,29 @@ void store_config() {
     glurp->config->refresh_rate = nr;
   }
 
+  w = glade_xml_get_widget(configxml, "checkbutton_save_size");
+  glurp->config->save_size = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+
   w = glade_xml_get_widget(configxml, "checkbutton_col_filename");
-  glurp->config->vis_filename = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_FILENAME] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_artist");
-  glurp->config->vis_artist = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_ARTIST] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_title");
-  glurp->config->vis_title = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_TITLE] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_album");
-  glurp->config->vis_album = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_ALBUM] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_name");
-  glurp->config->vis_name = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_NAME] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_pos");
-  glurp->config->vis_pos = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_POS] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 
   w = glade_xml_get_widget(configxml, "checkbutton_col_id");
-  glurp->config->vis_id = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
+  glurp->config->playlist_columns[PL_ID] = yesno(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 } 
 
 void create_playlist_liststore() {
@@ -340,8 +384,8 @@ void hide_gui_playlist() {
   gint width, height;
 
   gtk_widget_hide(glade_xml_get_widget(guixml, "vbox_playlist"));
-  gtk_window_get_size(GTK_WINDOW(glade_xml_get_widget(guixml, "window_main")), &width, &height);
-  gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(guixml, "window_main")), width, 1);
+  gtk_window_get_size(GTK_WINDOW(glade_xml_get_widget(guixml, "glurp_window_main")), &width, &height);
+  gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(guixml, "glurp_window_main")), width, 1);
 }
 
 void show_gui_playlist() {
@@ -392,24 +436,24 @@ void gui_set_connected() {
 
   if( status->repeat ) {
     debug("MPD repeat is ON");
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), TRUE);
   } else {
     debug("MPD repeat is OFF");
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), FALSE);
   }
 
   if( status->random ) {
     debug("MPD random is ON");
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), TRUE);
   } else {
     debug("MPD random is OFF");
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), FALSE);
   }
 
   gtk_widget_set_sensitive(glade_xml_get_widget(guixml, "volumebar"), TRUE);
 
   if( OUTPUTS_CAPABLE_MPD )
-    gtk_widget_show_all(glade_xml_get_widget(guixml, "toggletoolbutton_outputs"));
+    gtk_widget_show_all(glade_xml_get_widget(guixml, "togglebutton_outputs"));
 }    
 
 void gui_set_disconnected() {
@@ -430,10 +474,10 @@ void gui_set_disconnected() {
   gtk_widget_hide(glade_xml_get_widget(guixml, "button_pause"));
   gtk_widget_show(glade_xml_get_widget(guixml, "button_play"));
 
-  gtk_widget_hide(glade_xml_get_widget(guixml, "toggletoolbutton_outputs"));
+  gtk_widget_hide(glade_xml_get_widget(guixml, "togglebutton_outputs"));
 
-  gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), FALSE);
-  gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), FALSE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), FALSE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), FALSE);
   clear_playlist();
   gtk_list_store_clear(glurp->gui_playlist);
   glurp->playlist_version = -1;
@@ -454,7 +498,7 @@ gboolean gui_update(gint refresh_rate_status) {
   gchar *title;
   gboolean changed = FALSE, firstrun = FALSE;
 
-  if( DISCONNECTED ) {
+  if( DISCONNECTED || DISCONNECTING ) {
     debug("Resetting GUI to disconnected state.");
     glurp_disconnect();
     return FALSE;
@@ -559,16 +603,16 @@ gboolean gui_update(gint refresh_rate_status) {
 
 /* PERIODIC: update repeat button */
   if( status->repeat ) {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), TRUE);
   } else {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_repeat")), FALSE);
   }
 
 /* PERIODIC: update random button */
   if( status->random ) {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), TRUE);
   } else {
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(guixml, "togglebutton_random")), FALSE);
   }
 
   if( changed && CONNECTED && (PLAYING || PAUSED) ) {
@@ -855,6 +899,46 @@ void populate_gui_add_tree() {
   }
 }
 
+void populate_gui_add_search_tree() {
+  GtkComboBox* combo_type;
+  GtkEntry* find_entry;
+  GtkTreeIter iter;
+  GtkTreePath *path;
+  gint n, nd;
+  const gchar *what;
+
+  if (!glurp->gui_addtree) return;
+
+  /* remove last toplevel item if it's search results */
+  n = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(glurp->gui_addtree), NULL);
+  gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(glurp->gui_addtree), &iter, NULL, n - 1);
+  gtk_tree_model_get(GTK_TREE_MODEL(glurp->gui_addtree), &iter, ADD_NUM_DIRS, &nd, -1);
+  if( nd == -2 ) {	/* last item is search results, let's ixnay it */
+    debug("Removing previous search results treeview item");
+    gtk_tree_store_remove(glurp->gui_addtree, &iter);
+  }
+
+  combo_type = GTK_COMBO_BOX(glade_xml_get_widget(addxml, "combo_add_find_type"));
+
+  debug("Adding a folder for find results");
+  find_entry = GTK_ENTRY(glade_xml_get_widget(addxml, "entry_add_find_what"));
+  what = gtk_entry_get_text(find_entry);
+  if( what && g_utf8_strlen(what, -1) ) {
+    gtk_tree_store_append(glurp->gui_addtree, &iter, NULL);
+    gtk_tree_store_set(glurp->gui_addtree, &iter, ADD_ICON, gtk_widget_render_icon(glade_xml_get_widget(addxml, "glurp_window_add"), GTK_STOCK_FIND, GTK_ICON_SIZE_SMALL_TOOLBAR, "icon"), ADD_NAME, "Search results", ADD_NUM_DIRS, -2, -1);
+
+    path = gtk_tree_model_get_path(GTK_TREE_MODEL(glurp->gui_addtree), &iter);
+    glurp_add_search_result_dir( gtk_entry_get_text(find_entry), gtk_combo_box_get_active(combo_type), path);
+  }
+  
+  if( !glurp->updating_db ) {
+    debug("Setting 'Add' treeview model");
+    gtk_tree_view_set_model(GTK_TREE_VIEW(glade_xml_get_widget(addxml, "treeview_add")), GTK_TREE_MODEL(glurp->gui_addtree));
+  }
+
+  gtk_editable_select_region(GTK_EDITABLE(find_entry), 0, -1);
+}
+  
 void gui_load_selected() {
   GtkTreeIter iter;
   GtkTreeView *tv = GTK_TREE_VIEW(glade_xml_get_widget(addxml, "treeview_add"));
@@ -873,11 +957,8 @@ void gui_load_selected() {
   selected_rows = gtk_tree_selection_get_selected_rows(ts, NULL);
 
   for( i = 0; i < num; i++ ) {
-    debug("%d >>>>>>>", i);
     gtk_tree_model_get_iter(tm, &iter, (GtkTreePath *)g_list_nth_data(selected_rows, i));
-    debug("%d >>>>>>>", i);
     gui_load_song(tm, iter);
-    debug("%d >>>>>>>", i);
   }
 
   return;
@@ -1078,9 +1159,7 @@ void populate_stream_liststore() {
   GlurpStream *s = NULL;
   GtkTreeIter iter;
 
-  debug("foo");
   gtk_list_store_clear(glurp->gui_stream_list);
-  debug("foo");
 
   for( s = glurp->stream_history; s; s = s->next ) {
     debug("Adding row '%s'", s->url);
@@ -1128,43 +1207,43 @@ void gui_refresh_playlist_columns() {
 
   rend = gtk_cell_renderer_text_new();
 
-  if( glurp->config->vis_pos ) {
+  if( glurp->config->playlist_columns[PL_POS] ) {
     col = gtk_tree_view_column_new_with_attributes("#", rend, "text", PL_POS, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
   }
     
-  if( glurp->config->vis_id ) {
+  if( glurp->config->playlist_columns[PL_ID] ) {
     col = gtk_tree_view_column_new_with_attributes("Id", rend, "text", PL_ID, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
   }
     
-  if( glurp->config->vis_title ) {
+  if( glurp->config->playlist_columns[PL_TITLE] ) {
     col = gtk_tree_view_column_new_with_attributes("Title", rend, "text", PL_TITLE, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
   }
     
-  if( glurp->config->vis_name ) {
+  if( glurp->config->playlist_columns[PL_NAME] ) {
     col = gtk_tree_view_column_new_with_attributes("Name", rend, "text", PL_TITLE, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
   }
     
-  if( glurp->config->vis_artist ) {
+  if( glurp->config->playlist_columns[PL_ARTIST] ) {
     col = gtk_tree_view_column_new_with_attributes("Artist", rend, "text", PL_ARTIST, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
   }
     
-  if( glurp->config->vis_album ) {
+  if( glurp->config->playlist_columns[PL_ALBUM] ) {
     col = gtk_tree_view_column_new_with_attributes("Album", rend, "text", PL_ALBUM, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
   }
     
-  if( glurp->config->vis_filename ) {
+  if( glurp->config->playlist_columns[PL_FILENAME] ) {
     col = gtk_tree_view_column_new_with_attributes("Filename", rend, "text", PL_FILENAME, "weight", PL_BOLD, "weight-set", PL_BOLD_SET, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
     gtk_tree_view_append_column(tv, col);
@@ -1223,3 +1302,34 @@ GtkMenu *populate_outputs_menu() {
 
   return GTK_MENU(menu);
 }
+
+void gui_add_append(gchar *path, GtkTreePath *gpath, gboolean song) {
+  GtkTreeModel *tm = GTK_TREE_MODEL(glurp->gui_addtree);
+  GtkTreeIter iparent, iter, newchild;
+  gint nd;
+
+  debug("Adding '%s'", path);
+
+  /* get stored number of dirs in parent dir */
+  if( gpath ) {
+    gtk_tree_model_get_iter(tm, &iparent, gpath);
+    gtk_tree_model_get(tm, &iparent, ADD_NUM_DIRS, &nd, -1);
+  }
+
+  if( song ) {
+    gtk_tree_store_append(glurp->gui_addtree, &iter, (gpath ? &iparent : NULL));
+    gtk_tree_store_set(glurp->gui_addtree, &iter, 3, gdk_pixbuf_new_from_file(DATADIR "pixmaps/media-audiofile.png", NULL), -1);
+  } else {
+    gtk_tree_store_insert(glurp->gui_addtree, &iter, (gpath ? &iparent : NULL), (gpath ? nd : glurp->num_add_dirs));
+    if( gpath ) gtk_tree_store_set(glurp->gui_addtree, &iparent, 2, nd+1, -1);
+    else glurp->num_add_dirs++;
+    gtk_tree_store_set(glurp->gui_addtree, &iter, 3, gtk_widget_render_icon(glade_xml_get_widget(guixml, "glurp_window_main"), GTK_STOCK_OPEN, GTK_ICON_SIZE_SMALL_TOOLBAR, "icon"), -1);
+    gtk_tree_store_set(glurp->gui_addtree, &iter, ADD_NUM_DIRS, -1, -1);
+    gtk_tree_store_insert(glurp->gui_addtree, &newchild, &iter, (gpath ? nd : glurp->num_add_dirs));
+  }
+
+  gtk_tree_store_set(glurp->gui_addtree, &iter, ADD_NAME, glurp_filename(path), ADD_FILENAME, path, -1);
+
+  if( gpath )
+    gtk_tree_view_expand_row(GTK_TREE_VIEW(glade_xml_get_widget(addxml, "treeview_add")), gpath, FALSE);
+}
